/*
 * nodejs-license-file
 * https://github.com/bushev/nodejs-license-file
 *
 * Copyright (c) 2016 Yuriy Bushev
 * Licensed under the MIT license.
 */

'use strict';

/**
 * Core fs module
 *
 * @type {exports|module.exports}
 */
const fs = require('fs');

/**
 * Core path module
 *
 * @type {posix|exports|module.exports}
 */
const path = require('path');

/**
 * Core crypto module
 *
 * @type {exports|module.exports}
 */
const crypto = require('crypto');

/**
 * Default license file template
 */
const defaultTemplate = fs.readFileSync(path.resolve(__dirname, '..', 'data', 'default.tpl'), 'utf8');

/**
 * LicenseFile Class
 */
class LicenseFile {

    /**
     *  Generate license file
     *
     * @param options
     * @param options.data {object|string} - data to sign
     * @param [options.privateKeyPath] {string} - path to private key
     * @param [options.privateKey] {string} - private key content
     * @param [options.template] - custom license file template
     * @param callback {function} - callback function
     */
    static generate(options, callback) {

        let privateKey;

        if (!options.data) {
            return callback(new Error('LicenseFile::generate: options.data is required'));
        }

        if (typeof options.privateKey === 'string') {

            privateKey = options.privateKey;

        } else if (typeof options.privateKeyPath === 'string') {

            privateKey = fs.readFileSync(options.privateKeyPath, 'utf8');

        } else {

            return callback(new Error('LicenseFile::generate: privateKeyPath or privateKey is required'));
        }

        if (typeof callback !== 'function') {
            return callback(new Error('LicenseFile::parse: callback is required'));
        }

        const template = (typeof options.template === 'string') ? options.template : defaultTemplate;

        const serial = LicenseFile._generateSerial(Object.assign({}, options, {privateKey}));

        if (typeof options.data === 'string') {
            options.data = {string: options.data};
        }

        options.data.serial = serial;

        callback(null, LicenseFile._render(template, LicenseFile._prepareDataObject(options.data)));
    }

    /**
     * Parse license file
     *
     * @param options
     * @param [options.publicKeyPath] {string} - path to public key
     * @param [options.publicKey] {string} - path to public key
     * @param options.fileData {string} - license file content
     * @param [options.fileParseFnc] {function} - file parse function
     * @param callback
     */
    static parse(options, callback) {

        let publicKey;

        if (typeof options.publicKey === 'string') {

            publicKey = options.publicKey;

        } else if (typeof options.publicKeyPath === 'string') {

            publicKey = fs.readFileSync(options.publicKeyPath, 'utf8');

        } else {

            return callback(new Error('LicenseFile::parse: publicKeyPath or publicKey is required'));
        }

        if (typeof options.fileData !== 'string') {
            return callback(new Error('LicenseFile::parse: options.fileData is required'));
        }

        if (typeof options.fileParseFnc !== 'function') {
            options.fileParseFnc = LicenseFile._defaultFileParseFnc;
        }

        if (typeof callback !== 'function') {
            return callback(new Error('LicenseFile::parse: callback is required'));
        }

        options.fileParseFnc(options.fileData, (err, parsedData) => {
            if (err) return callback(err);

            if (typeof parsedData.serial !== 'string') {
                return callback(new Error('LicenseFile::fileParseFnc: serial string was not passed to callback'));
            }

            if (typeof parsedData.data !== 'string' && typeof parsedData.data !== 'object') {
                return callback(new Error('LicenseFile::fileParseFnc: data string/object was not passed to callback'));
            }

            let data     = parsedData.data;
            const verify = crypto.createVerify('RSA-SHA256');

            if (typeof data === 'object') {
                data = JSON.stringify(data);
            }

            verify.update(data);

            const valid = verify.verify(publicKey, parsedData.serial, 'base64');

            callback(null, {valid: valid, serial: parsedData.serial, data: parsedData.data});
        });
    }

    /**
     *
     * @param options
     * @param options.data
     * @param options.privateKey
     * @private
     */
    static _generateSerial(options) {

        const sign = crypto.createSign('RSA-SHA256');
        let data   = options.data;

        if (typeof options.data === 'object') {
            data = JSON.stringify(LicenseFile._prepareDataObject(options.data));
        }

        sign.update(data);

        return sign.sign(options.privateKey, 'base64');
    }

    static _defaultFileParseFnc(fileData, callback) {

        const dataLines = fileData.split('\n');

        if (dataLines.length !== 4) {
            return callback(new Error('LicenseFile::_defaultFileParseFnc: License file must have 4 lines'));
        }

        const data   = dataLines[1];
        const serial = dataLines[2];

        callback(null, {serial: serial, data: data});
    }

    static _render(template, data) {

        for (let property in data) {

            if (data.hasOwnProperty(property)) {

                const regExp = new RegExp(`{{&${property}}}`, `g`);

                template = template.replace(regExp, data[property]);
            }
        }

        return template;
    }

    static _prepareDataObject(data) {

        const result = {};

        for (let property in data) {

            if (data.hasOwnProperty(property)) {

                result[property] = typeof data[property] === 'string' ? data[property] : data[property] + '';
            }
        }

        return result;
    }
}

/**
 * Export LicenseFile Class
 *
 * @type {LicenseFile}
 */
module.exports = LicenseFile;
